/*
 (C) Tomonori Izumi <izumi@ieee.org>, Sep. 2015. All rigts reserved.
 for xc7z020clg484-1 on Digilent ZYBO
 					Ver.2015.09.28a
*/
 
`timescale 1ns / 1ps

module tb_top();
   reg m0rexec,m1rexec,s0wexec,s1wexec;
   reg w0wexec,w1wexec,r0rexec,r1rexec;
   reg wexec,rexec;
   reg run;
   reg [15:0] i;
   reg 	tmp=0;

   wire	clk;
   wire	rstn;

   reg 	arm_clko=1;
   reg 	arm_rstno=0;
   reg [31:0] arm_axi0_awaddr;
   reg [2:0]  arm_axi0_awprot;
   reg [0:0]  arm_axi0_awvalid;
   wire [0:0] arm_axi0_awready;
   reg [31:0] arm_axi0_wdata;
   reg [3:0]  arm_axi0_wstrb;
   reg [0:0]  arm_axi0_wvalid;
   wire [0:0] arm_axi0_wready;
   wire [1:0]  arm_axi0_bresp;
   wire [0:0]  arm_axi0_bvalid;
   reg [0:0]  arm_axi0_bready;
      
   reg [31:0] arm_axi0_araddr;
   reg [2:0]  arm_axi0_arprot;
   reg [0:0]  arm_axi0_arvalid;
   wire [0:0] arm_axi0_arready;
   wire [31:0] arm_axi0_rdata;
   wire [1:0]  arm_axi0_rresp;
   wire [0:0]  arm_axi0_rvalid;
   reg [0:0]   arm_axi0_rready;

   always #5 arm_clko = ~arm_clko;
   initial begin
      arm_clko=1;
      arm_rstno=0;
      run=1;
      #40 arm_rstno=1;
   end
   assign 	clk=arm_clko;
   assign 	rstn=arm_rstno;
   
   //----------------------------------------------------------------
   // setup : FIFO from AXI-Lite Bus

   parameter 	DW=16;
   parameter 	AW=3;
   parameter 	SZ=8;
   
   wire [DW-1:0] 	m0d,m1d;
   reg [DW-1:0] 	s0d,s1d;
   wire 		m0v,m1v,s0r,s1r;
   reg 			m0r,m1r,s0v,s1v;
   wire [AW:0] 		m0a,m1a,s0a,s1a;
   
   defparam 	axififo_i.M0MD=3;
   defparam 	axififo_i.M0DW=DW;
   defparam 	axififo_i.M0AW=AW;
   defparam 	axififo_i.M0SZ=SZ;
   defparam 	axififo_i.M1MD=3;
   defparam 	axififo_i.M1DW=DW;
   defparam 	axififo_i.M1AW=AW;
   defparam 	axififo_i.M1SZ=SZ;
   defparam 	axififo_i.S0MD=3;
   defparam 	axififo_i.S0DW=DW;
   defparam 	axififo_i.S0AW=AW;
   defparam 	axififo_i.S0SZ=SZ;
   defparam 	axififo_i.S1MD=3;
   defparam 	axififo_i.S1DW=DW;
   defparam 	axififo_i.S1AW=AW;
   defparam 	axififo_i.S1SZ=SZ;
   rizm_axilite_fifo axififo_i
     (.s_axi_aclk(arm_clko),
      .s_axi_aresetn(arm_rstno),
      .s_axi_awaddr(arm_axi0_awaddr),
      .s_axi_awvalid(arm_axi0_awvalid),
      .s_axi_awready(arm_axi0_awready),
      .s_axi_wdata(arm_axi0_wdata),
      .s_axi_wstrb(arm_axi0_wstrb),
      .s_axi_wvalid(arm_axi0_wvalid),
      .s_axi_wready(arm_axi0_wready),
      .s_axi_bresp(arm_axi0_bresp),
      .s_axi_bvalid(arm_axi0_bvalid),
      .s_axi_bready(arm_axi0_bready),
      .s_axi_araddr(arm_axi0_araddr),
      .s_axi_arvalid(arm_axi0_arvalid),
      .s_axi_arready(arm_axi0_arready),
      .s_axi_rdata(arm_axi0_rdata),
      .s_axi_rresp(arm_axi0_rresp),
      .s_axi_rvalid(arm_axi0_rvalid),
      .s_axi_rready(arm_axi0_rready),
      .m0_axis_tdata( m0d),
      .m0_axis_tvalid(m0v),
      .m0_axis_tready(m0r),
      .m0_axis_ravail(m0a),
      .m1_axis_tdata( m1d),
      .m1_axis_tvalid(m1v),
      .m1_axis_tready(m1r),
      .m1_axis_ravail(m1a),
      .s0_axis_tdata( s0d),
      .s0_axis_tvalid(s0v),
      .s0_axis_tready(s0r),
      .s0_axis_wavail(s0a),
      .s1_axis_tdata( s1d),
      .s1_axis_tvalid(s1v),
      .s1_axis_tready(s1r),
      .s1_axis_wavail(s1a));

   always @(posedge clk) begin
      m0r<=(m0rexec)?1:0;
      m1r<=(m1rexec)?1:0;
      s0v<=(s0wexec)?1:0;
      s1v<=(s1wexec)?1:0;
      if (s0wexec&s0r) s0d<=s0d+1;
      if (s1wexec&s1r) s1d<=s1d+1;
   end
      

   reg [DW-1:0] w0d,w1d,r0d,r1d;
   reg [31:0] wbuf,rbuf;
   reg [7:0]  baseaddr;

   initial begin
      #3 tmp=~tmp;

      arm_axi0_awvalid=0;
      arm_axi0_wvalid=0;
      arm_axi0_bready=0;
      m0r=0;
      m1r=0;
      s0v=0;
      s1v=0;
      s0d=16'h2000;
      s1d=16'h3000;
      
      w0wexec=0;
      w1wexec=0;
      r0rexec=0;
      r1rexec=0;

      m0rexec=0;
      m1rexec=0;
      s0wexec=0;
      s1wexec=1;

      arm_axi0_arvalid=0;
      arm_axi0_rready=0;

      while (rstn==0) begin
	 #10 tmp=~tmp;
      end

      #250 tmp=~tmp;

      arm_axi0_wstrb=15;
      arm_axi0_awprot=0;
      arm_axi0_arprot=0;
      arm_axi0_wdata=0;
      arm_axi0_awaddr=0;
      arm_axi0_araddr=0;
      wbuf=0;
      rbuf=0;
      baseaddr=0;
      wexec=0;
      rexec=0;
      w0d=16'h4000;
      w1d=16'h5000;
      i=0;
      w0wexec=1;
      w1wexec=1;
      r0rexec=1;
      r1rexec=1;

      #10 tmp=~tmp;

      while (i<32) begin
	 if (i==14) s1wexec=0;
	 if (0) begin
	    wbuf=w0d;
	    #10 baseaddr=8'h00;
	    w0d=w0d+1;
	    wexec=w0wexec;
	    rexec=0;
	 end else if (0) begin
	    #10 baseaddr=8'h20;
	    wexec=0;
	    rexec=r0rexec;
	 end else if (0) begin
	    wbuf=w1d;
	    #10 baseaddr=8'h10;
	    w1d=w1d+1;
	    wexec=w1wexec;
	    rexec=0;
	 end else if (1) begin
	    #10 baseaddr=8'h30;
	    wexec=0;
	    rexec=r1rexec;
	 end
	 

	 arm_axi0_araddr=baseaddr+4;
	 arm_axi0_arvalid=1;
	 arm_axi0_rready=1;
	 while (arm_axi0_rvalid==0) begin
	    #10 arm_axi0_rready=1;
	 end
	 rbuf=arm_axi0_rdata;
	 arm_axi0_arvalid=0;
	 #10 arm_axi0_rready=0;

	 #10 tmp=~tmp;
	 
	 arm_axi0_araddr=baseaddr+8;
	 arm_axi0_arvalid=1;
	 arm_axi0_rready=1;
	 while (arm_axi0_rvalid==0) begin
	    #10 arm_axi0_rready=1;
	 end
	 rbuf=arm_axi0_rdata;
	 arm_axi0_arvalid=0;
	 #10 arm_axi0_rready=0;

	 #10 tmp=~tmp;
	 
	 if (wexec) begin
	    arm_axi0_awaddr=baseaddr+0;
	    arm_axi0_awvalid=1;
	    arm_axi0_wdata=wbuf;
	    arm_axi0_wvalid=1;
	    arm_axi0_bready=1;
	    while (arm_axi0_bvalid==0) begin
	       #10 arm_axi0_bready=1;
	    end
	    arm_axi0_awvalid=0;
	    arm_axi0_wvalid=0;
	    #10 arm_axi0_bready=0;
	 end // if (wexec)

	 if (rexec) begin
	    arm_axi0_araddr=baseaddr+0;
	    arm_axi0_arvalid=1;
	    arm_axi0_rready=1;
	    while (arm_axi0_rvalid==0) begin
	       #10 arm_axi0_rready=1;
	    end
	    rbuf=arm_axi0_rdata;
	    if (baseaddr==8'h20) r0d=rbuf;
	    if (baseaddr==8'h30) r1d=rbuf;
	    arm_axi0_arvalid=0;
	    #10 arm_axi0_rready=0;
	 end // if (rexec)

	 i=i+1;
      end

      #160 $finish;
   end

   initial begin
      $dumpfile("rizm_axilite_fifo_tb.vcd");
      $dumpvars;
      #100000 $finish;
   end
   
endmodule

// [Synth 8-3331] unconnected port
// [Synth 8-3848] does not have driver
// [Synth 8-3295] tying undriven pin to constant
