#include <stdio.h>
#include <stdlib.h>
#include "platform.h"
#include "xparameters.h"
#include "xgpio.h"
#include "xgpiops.h" // for PS GPIO(MIO)
#include "xil_cache.h"

// yz
// n[hEFA@foȌݎ菇
// (1) {[hZbg(ZedBoard:PS-RST,ZYBO:PS-SRST)
// (2) Vivado  Program Device
// (3) SDK  Run As Hardware
// (4) Vivado  Reflesh device

/* Write Port 0 (AXI Lite -> Direct -> AXIS Master 0) */
/* Write Port 1 (AXI Lite ->  FIFO  -> AXIS Master 1) */
/* Read  Port 0 (AXI Lite <- Direct <- AXIS Slave  0) */
/* Read  Port 1 (AXI Lite <-  FIFO  <- AXIS Slave  1) */
volatile unsigned int *w0data_p =XPAR_RIZM_AXILITE_FIFO_0_BASEADDR+0x00;
volatile unsigned int *w0busy_p =XPAR_RIZM_AXILITE_FIFO_0_BASEADDR+0x04;
volatile unsigned int *w0avail_p=XPAR_RIZM_AXILITE_FIFO_0_BASEADDR+0x08;
volatile unsigned int *w1data_p =XPAR_RIZM_AXILITE_FIFO_0_BASEADDR+0x10;
volatile unsigned int *w1busy_p =XPAR_RIZM_AXILITE_FIFO_0_BASEADDR+0x14;
volatile unsigned int *w1avail_p=XPAR_RIZM_AXILITE_FIFO_0_BASEADDR+0x18;
volatile unsigned int *r0data_p =XPAR_RIZM_AXILITE_FIFO_0_BASEADDR+0x20;
volatile unsigned int *r0busy_p =XPAR_RIZM_AXILITE_FIFO_0_BASEADDR+0x24;
volatile unsigned int *r0avail_p=XPAR_RIZM_AXILITE_FIFO_0_BASEADDR+0x28;
volatile unsigned int *r1data_p =XPAR_RIZM_AXILITE_FIFO_0_BASEADDR+0x30;
volatile unsigned int *r1busy_p =XPAR_RIZM_AXILITE_FIFO_0_BASEADDR+0x34;
volatile unsigned int *r1avail_p=XPAR_RIZM_AXILITE_FIFO_0_BASEADDR+0x38;

volatile unsigned int *gpio0_p=XPAR_AXI_GPIO_0_BASEADDR+XGPIO_DATA_OFFSET;
volatile unsigned int *gpio1_p=XPAR_AXI_GPIO_0_BASEADDR+XGPIO_DATA2_OFFSET;

#define LED_PIN 7
#define PB1_PIN 50
#define PB2_PIN 51
static XGpioPs gpiops;

int main(){
  static XGpioPs_Config* gpiops_config_ptr;
  int pb1,pb2,pb1_prev,pb2_prev,led;
  int pl_btn,pl_sw,pl_btn_prev,pl_btn2;
  int w0d,w0b,w0a,w1d,w1b,w1a,r0d,r0b,r0a,r1d,r1b,r1a;
  int counter;
  int buf[256];
  int r,i,k;
  int c0=0,c1=0;
  init_platform();
  Xil_ICacheEnable();
  Xil_DCacheEnable();
  
  print("testfifo.c: ");
  print(__TIMESTAMP__);
  print("\r\n");
  
  gpiops_config_ptr=XGpioPs_LookupConfig(XPAR_XGPIOPS_0_DEVICE_ID);
  r=XGpioPs_CfgInitialize(&gpiops,gpiops_config_ptr,gpiops_config_ptr->BaseAddr);
  if(r!=XST_SUCCESS) {
    print( "GPIOps Init Err\n\r");
    exit(-1);
  }
  XGpioPs_SetDirectionPin   (&gpiops,LED_PIN,1);
  XGpioPs_SetDirectionPin   (&gpiops,PB1_PIN,0);
  XGpioPs_SetDirectionPin   (&gpiops,PB2_PIN,0);
  XGpioPs_SetOutputEnablePin(&gpiops,LED_PIN,1);
  XGpioPs_SetOutputEnablePin(&gpiops,PB1_PIN,0);
  XGpioPs_SetOutputEnablePin(&gpiops,PB2_PIN,0);
  
  pb1_prev=XGpioPs_ReadPin(&gpiops,PB1_PIN);
  pb2_prev=XGpioPs_ReadPin(&gpiops,PB2_PIN);
  pl_btn_prev=(*gpio0_p);
  counter=0;
  while(1){
    pb1=XGpioPs_ReadPin(&gpiops,PB1_PIN);
    pb2=XGpioPs_ReadPin(&gpiops,PB2_PIN);
    led=(counter&0x40000)?1:0;
    XGpioPs_WritePin(&gpiops,LED_PIN,led);
    pl_btn=(*gpio0_p);
    pl_sw =(*gpio1_p);
    pl_btn2=pl_btn&~pl_btn_prev; // 0to1 pulse
    switch (pl_btn2) {
    case 0x1: /* BTN0 ... Write Port 0 */
      w0b=(*w0busy_p);
      w0a=(*w0avail_p);
      c0=(c0+1)&15;
      w0d=c0;
      (*w0data_p)=w0d;
      printf("Write Port0 (busy %d,avail%3d) <-%3d data %1x\n",w0b,w0a,1,w0d);
      break;
    case 0x2: /* BTN1 ... Write Port 1 */
      w1b=(*w1busy_p);
      w1a=(*w1avail_p);
      if (pl_sw==0) {
	/* Write Single */
	c1=(c1+1)&31; 
	w1d=c1+' ';
	k=1;
	buf[0]=w1d;
	(*w1data_p)=w1d;
      } else { 
	/* Write Multiple */
	k=w1a-8+pl_sw;
	for (i=0;i<k;i++) buf[i]='@'+i;
	for (i=0;i<k;i++) (*w1data_p)=buf[i];
      }
      printf("Write Port1 (busy %d,avail%3d) <-%3d data ",w1b,w1a,k);
      for (i=0;i<k;i++) printf("%c",buf[i]);
      printf("\n");
      break;
    case 0x4: /* BTN2 ... Read Port 0 */
      r0b=(*r0busy_p);
      r0a=(*r0avail_p);
      r0d=(*r0data_p);
      printf("Read  Port0 (busy %d,avail%3d) ->%3d data %1x\n",r0b,r0a,1,r0d);
      break;
    case 0x8: /* BTN3 ... Read Port 1 */
      r1b=(*r1busy_p);
      r1a=(*r1avail_p);
      if (pl_sw==0) {
	/* Read Single */
	r1d=(*r1data_p);
	k=1;
	buf[0]=r1d;
      } else {
	/* Read Multiple */
	k=r1a-8+pl_sw;
	for (i=0;i<k;i++) buf[i]=(*r1data_p);
      }
      printf("Read  Port1 (busy %d,avail%3d) ->%3d data ",r1b,r1a,k);
      for (i=0;i<k;i++) printf("%c",buf[i]);
      printf("\n");
      break;
    }
    counter++;
    pb1_prev=pb1;
    pb2_prev=pb2;
    pl_btn_prev=pl_btn;
  }
  
  print("Exit Main\n");
  Xil_DCacheDisable();
  Xil_ICacheDisable();
  
  
  cleanup_platform();
  return 0;
}
