// (C) Tomonori Izumi <izumi@ieee.org>, Oct. 2015. All rigts reserved.

module rizm_axilite_fifo
  (s_axi_aclk,    s_axi_aresetn,
   s_axi_awaddr,                  s_axi_awvalid,  s_axi_awready,
   s_axi_wdata,   s_axi_wstrb,    s_axi_wvalid,   s_axi_wready,
   s_axi_bresp,                   s_axi_bvalid,   s_axi_bready,
   s_axi_araddr,                  s_axi_arvalid,  s_axi_arready,
   s_axi_rdata,   s_axi_rresp,    s_axi_rvalid,   s_axi_rready,
   m0_axis_tdata, m0_axis_tvalid, m0_axis_tready, m0_axis_ravail,
   m1_axis_tdata, m1_axis_tvalid, m1_axis_tready, m1_axis_ravail,
   s0_axis_tdata, s0_axis_tvalid, s0_axis_tready, s0_axis_wavail,
   s1_axis_tdata, s1_axis_tvalid, s1_axis_tready, s1_axis_wavail);

   parameter AXIAW = 8; // address width of AXI subspace
   parameter M0MD  = 3; // port mode of M0
   parameter M0DW  = 8; // data width of M0 out
   parameter M0AW  = 3; // address width of M0 FIFO
   parameter M0SZ  = 8; // depth of M0 FIFO (actual size = SZ+4)
   parameter M1MD  = 3; // port mode of M1
   parameter M1DW  = 8; // data width of M1 out
   parameter M1AW  = 3; // address width of M1 FIFO
   parameter M1SZ  = 8; // depth of M1 FIFO (actual size = SZ+4)
   parameter S0MD  = 3; // port mode of S0
   parameter S0DW  = 8; // data width of S0 in
   parameter S0AW  = 3; // address width of S0 FIFO
   parameter S0SZ  = 8; // depth of S0 FIFO (actual size = SZ+4)
   parameter S1MD  = 3; // port mode of S1
   parameter S1DW  = 8; // data width of S1 in
   parameter S1AW  = 3; // address width of S1 FIFO
   parameter S1SZ  = 8; // depth of S1 FIFO (actual size = SZ+4)
   // port mode ... 1:reg, 2:direct strm, 3:buffered strm, others:none
   // address width of FIFO ... minimum 3 
   
   parameter A_M0FIFODATA = 8'h00;
   parameter A_M0FIFOBUSY = 8'h04;
   parameter A_M0FIFOAVAIL= 8'h08;
   parameter A_M1FIFODATA = 8'h10;
   parameter A_M1FIFOBUSY = 8'h14;
   parameter A_M1FIFOAVAIL= 8'h18;
   parameter A_S0FIFODATA = 8'h20;
   parameter A_S0FIFOBUSY = 8'h24;
   parameter A_S0FIFOAVAIL= 8'h28;
   parameter A_S1FIFODATA = 8'h30;
   parameter A_S1FIFOBUSY = 8'h34;
   parameter A_S1FIFOAVAIL= 8'h38;
   
   input             s_axi_aclk;
   input             s_axi_aresetn;
   input [AXIAW-1:0] s_axi_awaddr;
   input 	     s_axi_awvalid;
   output 	     s_axi_awready;
   input [31:0]      s_axi_wdata;
   input [3:0] 	     s_axi_wstrb;
   input 	     s_axi_wvalid;
   output 	     s_axi_wready;
   output [1:0]      s_axi_bresp;
   output 	     s_axi_bvalid;
   input 	     s_axi_bready;
   input [AXIAW-1:0] s_axi_araddr;
   input 	     s_axi_arvalid;
   output 	     s_axi_arready;
   output [31:0]     s_axi_rdata;
   output [1:0]      s_axi_rresp;
   output 	     s_axi_rvalid;
   input 	     s_axi_rready;
   output [M0DW-1:0] m0_axis_tdata;
   output 	     m0_axis_tvalid;
   input 	     m0_axis_tready;
   output [M0AW:0]   m0_axis_ravail;
   output [M1DW-1:0] m1_axis_tdata;
   output 	     m1_axis_tvalid;
   input 	     m1_axis_tready;
   output [M1AW:0]   m1_axis_ravail;
   input [S0DW-1:0]  s0_axis_tdata;
   input 	     s0_axis_tvalid;
   output 	     s0_axis_tready;
   output [S0AW:0]   s0_axis_wavail;
   input [S1DW-1:0]  s1_axis_tdata;
   input 	     s1_axis_tvalid;
   output 	     s1_axis_tready;
   output [S1AW:0]   s1_axis_wavail;

   reg 		   s_axi_awready_r=0;
   reg 		   s_axi_wready_r=0;
   reg [1:0] 	   s_axi_bresp_r=0;
   reg 		   s_axi_bvalid_r=0;
   reg 		   s_axi_arready_r=0;
   reg [31:0]	   s_axi_rdata_r=0;
   reg [1:0] 	   s_axi_rresp_r=0;
   reg 		   s_axi_rvalid_r=0;
   assign 	   s_axi_awready = s_axi_awready_r;
   assign 	   s_axi_wready  = s_axi_wready_r;
   assign 	   s_axi_bresp   = s_axi_bresp_r;
   assign 	   s_axi_bvalid  = s_axi_bvalid_r;
   assign 	   s_axi_arready = s_axi_arready_r;
   assign	   s_axi_rdata   = s_axi_rdata_r;
   assign 	   s_axi_rresp   = s_axi_rresp_r;
   assign 	   s_axi_rvalid  = s_axi_rvalid_r;

   
   // ------------------------------------------------
   // M0 FIFO out
   wire [M0DW-1:0] m0fifoidata;
   wire [M0DW-1:0] m0fifoodata;
   wire 	   m0fifoivalid;
   wire 	   m0fifoiwait,m0fifoovalid,m0fifoowait;
   wire [M0AW:0]   m0fifoiavail,m0fifooavail;
   defparam 	   m0fifo_i.DW=M0DW;
   defparam 	   m0fifo_i.AW=M0AW;
   defparam 	   m0fifo_i.SZ=M0SZ;
   rizm_fifo_2t m0fifo_i
     (s_axi_aclk,s_axi_aresetn,
      m0fifoidata,m0fifoivalid,m0fifoiwait,m0fifoiavail,
      m0fifoodata,m0fifoovalid,m0fifoowait,m0fifooavail);
   
   // ------------------------------------------------
   // M1 FIFO out
   wire [M1DW-1:0] m1fifoidata;
   wire [M1DW-1:0] m1fifoodata;
   wire 	   m1fifoivalid;
   wire 	   m1fifoiwait,m1fifoovalid,m1fifoowait;
   wire [M1AW:0]   m1fifoiavail,m1fifooavail;
   defparam 	   m1fifo_i.DW=M1DW;
   defparam 	   m1fifo_i.AW=M1AW;
   defparam 	   m1fifo_i.SZ=M1SZ;
   rizm_fifo_2t m1fifo_i
     (s_axi_aclk,s_axi_aresetn,
      m1fifoidata,m1fifoivalid,m1fifoiwait,m1fifoiavail,
      m1fifoodata,m1fifoovalid,m1fifoowait,m1fifooavail);

   // ------------------------------------------------
   // S0 FIFO in
   wire [S0DW-1:0] s0fifoidata;
   wire [S0DW-1:0] s0fifoodata;
   wire 	   s0fifoowait;
   wire 	   s0fifoivalid,s0fifoiwait,s0fifoovalid;
   wire [S0AW:0]   s0fifoiavail,s0fifooavail;
   defparam 	   s0fifo_i.DW=S0DW;
   defparam 	   s0fifo_i.AW=S0AW;
   defparam 	   s0fifo_i.SZ=S0SZ;
   rizm_fifo_2t s0fifo_i
     (s_axi_aclk,s_axi_aresetn,
      s0fifoidata,s0fifoivalid,s0fifoiwait,s0fifoiavail,
      s0fifoodata,s0fifoovalid,s0fifoowait,s0fifooavail);

   // ------------------------------------------------
   // S1 FIFO in
   wire [S1DW-1:0] s1fifoidata;
   wire [S1DW-1:0] s1fifoodata;
   wire 	   s1fifoowait;
   wire 	   s1fifoivalid,s1fifoiwait,s1fifoovalid;
   wire [S1AW:0]   s1fifoiavail,s1fifooavail;
   defparam 	   s1fifo_i.DW=S1DW;
   defparam 	   s1fifo_i.AW=S1AW;
   defparam 	   s1fifo_i.SZ=S1SZ;
   rizm_fifo_2t s1fifo_i
     (s_axi_aclk,s_axi_aresetn,
      s1fifoidata,s1fifoivalid,s1fifoiwait,s1fifoiavail,
      s1fifoodata,s1fifoovalid,s1fifoowait,s1fifooavail);

   
   // ------------------------------------------------
   // state machine to control AXI-LITE slave
   parameter 	   STT_WIDLE  =2'b01;
   parameter 	   STT_WWAIT  =2'b10;
   parameter 	   STT_RIDLE  =2'b01;
   parameter 	   STT_RWAIT  =2'b10;
   reg [1:0] 	   wstate_r=STT_WIDLE;
   reg [1:0] 	   rstate_r=STT_RIDLE;

   wire 	   s_axi_wexec;
   assign 	   s_axi_wexec=
		   (wstate_r==STT_WIDLE&&
		    s_axi_awvalid==1&&s_axi_wvalid==1)?1:0;

   wire 	   s_axi_rexec;
   assign 	   s_axi_rexec=
		   (rstate_r==STT_RIDLE&&
		    s_axi_arvalid==1)?1:0;
   
   // ------------------------------------------------
   // temp buffer for ports

   reg [M0DW-1:0]  m0data=0;
   reg 		   m0valid=0;
   wire 	   m0ready;
   wire [M0AW:0]   m0avail;
   
   reg [M1DW-1:0]  m1data=0;
   reg 		   m1valid=0;
   wire 	   m1ready;
   wire [M1AW:0]   m1avail;
   
   wire [S0DW-1:0] s0data;
   wire 	   s0valid;
   reg 		   s0ready=0;
   wire [S0AW:0]   s0avail;
   
   wire [S1DW-1:0] s1data;
   wire 	   s1valid;
   reg 		   s1ready=0;
   wire [S1AW:0]   s1avail;
   
   always @(posedge s_axi_aclk)
     if (s_axi_aresetn==0) begin
	m0data<=0;
	m0valid<=0;
     end else if (s_axi_wexec==1&&
		  s_axi_awaddr==A_M0FIFODATA&&
		  m0ready==1) begin
	m0data <=(M0MD==0)?0:s_axi_wdata[M0DW-1:0];
	m0valid<=(M0MD==0)?0:1;
     end else begin
	m0valid<=0;
     end

   always @(posedge s_axi_aclk)
     if (s_axi_aresetn==0) begin
	m1data<=0;
	m1valid<=0;
     end else if (s_axi_wexec==1&&
		  s_axi_awaddr==A_M1FIFODATA&&
		  m1ready==1) begin
	m1data <=(M1MD==0)?0:s_axi_wdata[M1DW-1:0];
	m1valid<=(M1MD==0)?0:1;
     end else begin
	m1valid<=0;
     end

   always @(posedge s_axi_aclk)
     if (s_axi_aresetn==0) begin
	s0ready<=0;
     end else if (s_axi_rexec==1&&
		  s_axi_araddr==A_S0FIFODATA&&
		  s0valid==1) begin
	s0ready<=(S0MD==0)?0:1;
     end else begin
	s0ready<=0;
     end

   always @(posedge s_axi_aclk)
     if (s_axi_aresetn==0) begin
	s1ready<=0;
     end else if (s_axi_rexec==1&&
		  s_axi_araddr==A_S1FIFODATA&&
		  s1valid==1) begin
	s1ready<=(S1MD==0)?0:1;
     end else begin
	s1ready<=0;
     end

   always @(posedge s_axi_aclk)
     if (s_axi_aresetn==0) begin
	wstate_r<=STT_WIDLE;
	s_axi_awready_r<=0;
	s_axi_wready_r<=0;
	s_axi_bvalid_r<=0;
     end else
       case (wstate_r)
	 STT_WIDLE: if (s_axi_wexec) begin
	    s_axi_awready_r<=1;
	    s_axi_wready_r<=1;
	    s_axi_bvalid_r<=1;
	    wstate_r<=STT_WWAIT;
	 end
	 STT_WWAIT: begin
	    s_axi_awready_r<=0;
	    s_axi_wready_r<=0;
	    if (s_axi_bready==1) begin
	       s_axi_bvalid_r<=0;
	       wstate_r<=STT_WIDLE;
	    end
	 end
       endcase // case(wstate_r)
   
   always @(posedge s_axi_aclk)
     if (s_axi_aresetn==0) begin
	rstate_r<=STT_RIDLE;
	s_axi_arready_r<=0;
	s_axi_rvalid_r<=0;
	s_axi_rdata_r<=0;
     end else
       case (rstate_r)
	 STT_RIDLE:
	   if (s_axi_rexec) begin
	      case (s_axi_araddr)
		A_S0FIFODATA:  s_axi_rdata_r<=(S0MD==0)?0: s0data; // trickey timing
		A_S1FIFODATA:  s_axi_rdata_r<=(S1MD==0)?0: s1data; // trickey timing
		A_M0FIFOBUSY:  s_axi_rdata_r<=(M0MD==0)?1:m0ready?0:1;
		A_M0FIFOAVAIL: s_axi_rdata_r<=(M0MD==0)?0:m0avail;
		A_M1FIFOBUSY:  s_axi_rdata_r<=(M1MD==0)?1:m1ready?0:1;
		A_M1FIFOAVAIL: s_axi_rdata_r<=(M1MD==0)?0:m1avail;
		A_S0FIFOBUSY:  s_axi_rdata_r<=(S0MD==0)?1:s0valid?0:1;
		A_S0FIFOAVAIL: s_axi_rdata_r<=(S0MD==0)?0:s0avail;
		A_S1FIFOBUSY:  s_axi_rdata_r<=(S1MD==0)?1:s1valid?0:1;
		A_S1FIFOAVAIL: s_axi_rdata_r<=(S1MD==0)?0:s1avail;
		default:       s_axi_rdata_r<=0;
	      endcase // case(s_axi_araddr)
	      s_axi_arready_r<=1;
	      s_axi_rvalid_r<=1;
	      rstate_r<=STT_RWAIT;
	   end
	 STT_RWAIT: begin
	    s_axi_arready_r<=0;
	    if (s_axi_rready==1) begin
	       s_axi_rvalid_r<=0;
	       rstate_r<=STT_RIDLE;
	    end
	 end
       endcase


   // ------------------------------------------------
   // connection for M0

   assign m0ready=
	  (M0MD==1)? 1: // register
	  (M0MD==2)? m0_axis_tready: // direct stream
	  (M0MD==3)?~m0fifoiwait: // buffered stream
	  0; // not connected
   assign m0avail=
	  (M0MD==1)? 1: // register
	  (M0MD==2)? m0_axis_tready: // direct stream
	  (M0MD==3)? m0fifoiavail: // buffered stream
	  0; // not connected

   assign m0fifoidata =(M0MD==3)? m0data:0;
   assign m0fifoivalid=(M0MD==3)? m0valid:0;
   assign m0fifoowait =(M0MD==3)?~m0_axis_tready:1;
   
   assign m0_axis_tdata=
	  (M0MD==1)? m0data: // register
	  (M0MD==2)? m0data: // direct stream
	  (M0MD==3)? m0fifoodata: // buffered stream
	  0; // not connected
   assign m0_axis_tvalid=
	  (M0MD==1)? 1: // register
	  (M0MD==2)? m0valid: // direct stream
	  (M0MD==3)? m0fifoovalid: // buffered stream
	  0; // not connected
   assign m0_axis_ravail=
	  (M0MD==1)? 1: // register
	  (M0MD==2)? m0valid: // direct stream
	  (M0MD==3)? m0fifooavail:
	  0; // not connected
   
   // ------------------------------------------------
   // connection for M1

   assign m1ready=
	  (M1MD==1)? 1: // register
	  (M1MD==2)? m1_axis_tready: // direct stream
	  (M1MD==3)?~m1fifoiwait: // buffered stream
	  0; // not connected
   assign m1avail=
	  (M1MD==1)? 1: // register
	  (M1MD==2)? m1_axis_tready: // direct stream
	  (M1MD==3)? m1fifoiavail: // buffered stream
	  0; // not connected

   assign m1fifoidata =(M1MD==3)? m1data:0;
   assign m1fifoivalid=(M1MD==3)? m1valid:0;
   assign m1fifoowait =(M1MD==3)?~m1_axis_tready:1;
   
   assign m1_axis_tdata=
	  (M1MD==1)? m1data: // register
	  (M1MD==2)? m1data: // direct stream
	  (M1MD==3)? m1fifoodata: // buffered stream
	  0; // not connected
   assign m1_axis_tvalid=
	  (M1MD==1)? 1: // register
	  (M1MD==2)? m1valid: // direct stream
	  (M1MD==3)? m1fifoovalid: // buffered stream
	  0; // not connected
   assign m1_axis_ravail=
	  (M1MD==1)? 1: // register
	  (M1MD==2)? m1valid: // direct stream
	  (M1MD==3)? m1fifooavail:
	  0; // not connected
   
   // ------------------------------------------------
   // connection for S0

   assign s0data=
	  (S0MD==1)? s0_axis_tdata: // register
	  (S0MD==2)? s0_axis_tdata: // direct stream
	  (S0MD==3)? s0fifoodata: // buffered stream
	  0; // not connected
   assign s0valid=
	  (S0MD==1)? 1: // register
	  (S0MD==2)? s0_axis_tvalid: // direct stream
	  (S0MD==3)? s0fifoovalid: // buffered stream
	  0; // not connected
   assign s0avail=
	  (S0MD==1)? 1: // register
	  (S0MD==2)? s0_axis_tvalid: // direct stream
	  (S0MD==3)? s0fifooavail: // buffered stream
	  0; // not connected

   assign s0fifoidata =(S0MD==3)? s0_axis_tdata:0;
   assign s0fifoivalid=(S0MD==3)? s0_axis_tvalid:0;
   assign s0fifoowait =(S0MD==3)?~s0ready:1;

   assign s0_axis_tready=
	  (S0MD==1)? 1: // register
	  (S0MD==2)? s0ready: // direct stream
	  (S0MD==3)?~s0fifoiwait: // buffered stream
	  0; // not connected
   assign s0_axis_wavail=
	  (S0MD==1)? 1: // register
	  (S0MD==2)? s0ready: // direct stream
	  (S0MD==3)? s0fifoiavail: // buffered stream
	  0; // not connected

   // ------------------------------------------------
   // connection for S1

   assign s1data=
	  (S1MD==1)? s1_axis_tdata: // register
	  (S1MD==2)? s1_axis_tdata: // direct stream
	  (S1MD==3)? s1fifoodata: // buffered stream
	  0; // not connected
   assign s1valid=
	  (S1MD==1)? 1: // register
	  (S1MD==2)? s1_axis_tvalid: // direct stream
	  (S1MD==3)? s1fifoovalid: // buffered stream
	  0; // not connected
   assign s1avail=
	  (S1MD==1)? 1: // register
	  (S1MD==2)? s1_axis_tvalid: // direct stream
	  (S1MD==3)? s1fifooavail: // buffered stream
	  0; // not connected

   assign s1fifoidata =(S1MD==3)? s1_axis_tdata:0;
   assign s1fifoivalid=(S1MD==3)? s1_axis_tvalid:0;
   assign s1fifoowait =(S1MD==3)?~s1ready:1;

   assign s1_axis_tready=
	  (S1MD==1)? 1: // register
	  (S1MD==2)? s1ready: // direct stream
	  (S1MD==3)?~s1fifoiwait: // buffered stream
	  0; // not connected
   assign s1_axis_wavail=
	  (S1MD==1)? 1: // register
	  (S1MD==2)? s1ready: // direct stream
	  (S1MD==3)? s1fifoiavail: // buffered stream
	  0; // not connected

endmodule

// model ... 1port ,pipelined, 2cycles delay
// parameter DW ... data bitwidth
// parameter AW ... addr bitwidth
// parameter SZ ... memory size (i.e. 2^AW)

module rizm_fifo_2t
  (clk,resetn,
   idata,ivalid,iwait,iavail,
   odata,ovalid,owait,oavail);
   parameter DW = 8;
   parameter AW = 3;
   parameter SZ = 8;
   input     clk;
   input     resetn;
   input [DW-1:0] idata;
   input 	  ivalid;
   output 	  iwait;
   output [DW-1:0] odata;
   output 	   ovalid;
   input 	   owait;
   output [AW:0]   iavail,oavail; // max SZ+4

   reg 		   iwait_r;
   assign 	   iwait=iwait_r;

   wire 	   iexec,oexec;
   assign 	   iexec = ivalid&~iwait;
   assign 	   oexec = ovalid&~owait;
   
   reg [AW:0] 	   iavail_r=SZ+4,oavail_r=0;
   assign 	   iavail=iavail_r;
   assign 	   oavail=oavail_r;
   always @(posedge clk)
     if (!resetn) begin
	iavail_r<=SZ+4;
	oavail_r<=0;
     end else if (iexec&~oexec) begin
	iavail_r<=iavail_r-1;
	oavail_r<=oavail_r+1;
     end else if (oexec&~iexec) begin
	iavail_r<=iavail_r+1;
	oavail_r<=oavail_r-1;
     end
   
   reg [AW-1:0]    raddr_r=0, waddr_r=0;
   reg [AW  :0]    memcount_r=0;
   
   wire [AW-1:0]   mema;
   wire [DW-1:0]   memd, memq;
   wire 	   memw_exe, memr_exe;
   reg 		   mv1=0,mv2=0; // delayed valid for pipelined read 
   
   defparam 	   mem.DW=DW;
   defparam 	   mem.AW=AW;
   defparam 	   mem.SZ=SZ;
   rizm_mem2pipe mem(clk,mema,memd,memq,memw_exe); // 1cyc pipe mem

   wire [2:0] 	   fifo4count;
   wire 	   fifo4iwait;
   defparam 	   fifo.DW=DW;
   rizm_fifo4reg fifo
     (clk,resetn,memq,mv2,fifo4iwait,odata,ovalid,owait,fifo4count);

   wire [2:0] 	   outbufcount;
   assign 	   outbufcount = fifo4count+mv1+mv2;

   assign memw_exe = ((iwait==0)&&(ivalid==1))?1:0;
   assign memr_exe = ((memw_exe==0)&&(memcount_r>0)&&(outbufcount<4))?1:0;
   
   assign mema     = memw_exe ? waddr_r : raddr_r;
   assign memd     = idata;

   always @(posedge clk)
     if (!resetn) begin
	waddr_r<=0;
	raddr_r<=0;
	memcount_r<=0;
	mv1<=0;
	mv2<=0;
	iwait_r<=1;
     end else begin
	if (memw_exe) begin
	   waddr_r<=waddr_r+1;
	   memcount_r<=memcount_r+1;
	   iwait_r<=1;
	   mv1<=0;
	end else if (memr_exe) begin
	   raddr_r<=raddr_r+1;
	   memcount_r<=memcount_r-1;
	   iwait_r<=0;
	   mv1<=1;
	end else begin
	   mv1<=0;
	   iwait_r<=(memcount_r<SZ)?0:1;
	end
	mv2<=mv1;
     end
   
endmodule

//----------------------------------------------------------------

module rizm_fifo4reg
  (clk,resetn,
   idata, ivalid, iwait,
   odata, ovalid, owait,
   count);
   parameter DW = 8;
   input     clk;
   input     resetn;
   input [DW-1:0] idata;
   input 	  ivalid;
   output 	  iwait;
   output [DW-1:0] odata;
   output 	   ovalid;
   input 	   owait;
   output [2:0]    count;

   reg 		   iwait_r;
   reg 		   ovalid_r;
   reg [DW-1:0]    r[3:0];
   reg [1:0] 	   waddr_r=0,raddr_r=0;
   reg [2:0] 	   count_r=0;
   assign 	   iwait=iwait_r;
   assign 	   ovalid=ovalid_r;
   assign odata = r[raddr_r];
   assign count = count_r;

   wire   iexec, oexec;
   assign iexec = ivalid&~iwait;
   assign oexec = ovalid&~owait;
   
   always @(posedge clk)
     if (!resetn) begin
	waddr_r<=0;
	raddr_r<=0;
	count_r<=0;
	iwait_r<=1;
	ovalid_r<=0;
     end else begin
	case ({iexec,oexec})
	  2'b11 : begin
	     r[waddr_r]<=idata;
	     waddr_r   <=waddr_r+1;
	     raddr_r   <=raddr_r+1;
	     ovalid_r  <=(count_r==0)?0:1;
	     iwait_r   <=(count_r==4)?1:0;
	  end
	  2'b10 : begin
	     r[waddr_r]<=idata;
	     waddr_r   <=waddr_r+1;
	     count_r   <=count_r+1;
	     iwait_r   <=(count_r==3)?1:0;
	     ovalid_r  <=1;
	  end
	  2'b01 : begin
	     raddr_r   <=raddr_r+1;
	     count_r   <=count_r-1;
	     iwait_r   <=0;
	     ovalid_r  <=(count_r==1)?0:1;
	  end
	  default : begin
	     iwait_r   <=(count_r==4)?1:0;
	     ovalid_r  <=(count_r==0)?0:1;
	  end
	endcase
     end
   
endmodule

//----------------------------------------------------------------
// block RAM model : 1port, pipelined, 2cycles delay
// parameter DW ... data bitwidth
// parameter AW ... addr bitwidth
// parameter SZ ... memory size (i.e. 2^AW)

module rizm_mem2pipe(clk, a, d, q, we);
   parameter DW = 8;
   parameter AW = 3;
   parameter SZ = 8;

   input     clk, we;
   input [AW-1:0] a;
   input [DW-1:0] d;
   output [DW-1:0] q;

   reg [AW-1:0]    a_r=0;
   reg [DW-1:0]    d_r=0;
   reg 		   we_r=0;
   reg [DW-1:0]    q_r=0;
   
   reg [DW-1:0]    mem[SZ-1:0];
   
   always @(posedge clk)
     begin
	a_r<=a;
	d_r<=d;
	we_r<=we;
     end

   always @(posedge clk)
     begin
	if (we_r) begin
	   mem[a_r]<=d_r;
	   q_r<=mem[a_r];
	end else begin
	   q_r<=mem[a_r];
	end
     end

   assign q = q_r;

endmodule

